#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdbool.h>

#define F_CPU 16000000ULL
#include <util/delay.h>

#include "can.h"

void sleep(uint32_t i)
{
  while(i--)
    _delay_ms(1);
}

int main(void)
{
  int8_t  data[8];
  uint8_t length;

  data[0] = 'H';
  data[1] = 'i';
  data[2] = ' ';
  data[3] = 't';
  data[4] = 'h';
  data[5] = 'e';
  data[6] = 'r';
  data[7] = 'e';

  // baudrate: 250K, keine Interrupts, extended Id
  can_init(CAN_BAUDRATE_250K, CAN_INTERRUPT_NONE, 1);

  can_enable_mob(0, CAN_MODE_TRANSMIT_DATA,   3, 0xffffffff);
  can_enable_mob(1, CAN_MODE_RECEIVE_DATA,    4, 0xffffffff);
  can_enable_mob(2, CAN_MODE_AUTO_REPLY,      5, 0xffffffff);

  // SENDEN:
  // wir senden insgesamt 8 frames mit der laenge 1 bis 8 mit der id 3 (mob 0).
  // was ankommen sollte:
  // "H"        (DLC: 1, ID: 3)
  // "Hi"       (DLC: 2, ID: 3)
  // "Hi "      (DLC: 3, ID: 3)
  // "Hi t"     (DLC: 4, ID: 3)
  // "Hi th"    (DLC: 5, ID: 3)
  // "Hi the"   (DLC: 6, ID: 3)
  // "Hi ther"  (DLC: 7, ID: 3)
  // "Hi there" (DLC: 8, ID: 3)
  for(length = 1; length <= 8; length++)
  {
    sleep(1000);
    can_send_data(0, length, data);
  }

  // REMOTE FRAME SENDEN:
  can_send_remote(1);

  // RECEIVE:
  // antworten werden in dem Interrupt-Handler in can.c abgearbeitet. Mom. muss
  // der Code als da rein. Wenn du eine Nachricht mit der Id 4 verschickst (von
  // einem anderen Bus-Teilnehmer) sollte der Interrupt-Handler in can.c
  // aufgerufen werden.

  // AUTOREPLY:
  // wenn ein anderer Bus-Teilnehmer ein Remote-Frame mit der id 6 schickt,
  // sollten er von uns die Antwort "Hi" bekommen. Das macht dann auch der
  // CAN-Controller ohne Hilfe des Microcontrollers
  can_set_autoreply(3, 2, data); // parameter: mob, laenge, daten

  while(1);

  return 0;
}
